# coding=utf-8
from django.views.generic import View
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseNotAllowed
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F
from django.views.generic import FormView
from django.shortcuts import get_object_or_404, render
from django.http import HttpResponsePermanentRedirect
from django.contrib.sites.models import Site
from django.views.decorators.csrf import csrf_exempt

from .models import URL, Visit, API_IP
from .forms import URLForm
from .piwik import response_cookie
from json import dumps, loads
from datetime import datetime, timedelta
import re

CURRENT_DOMAIN = Site.objects.get_current().domain
piwik_id = re.compile(r'^_pk_id')
site_url = re.compile(r'^http://' + CURRENT_DOMAIN + r'/(?P<slug>[-\w]+)$')


class JSONError(ValueError):
    pass


def json_esponse(dct):
        return HttpResponse(dumps(dct), content_type='application/json')


class JSONMixin(View):
    paginate_by = 20

    def dispatch(self, request, *args, **kwargs):
        try:
            log = API_IP.objects.get(remote_ip=request.META['REMOTE_ADDR'], start__gt=datetime.now() - timedelta(minutes=120))
        except API_IP.DoesNotExist:
            log = API_IP(remote_ip=request.META['REMOTE_ADDR'])
            log.save()

        if log.count > 30:
            return HttpResponseBadRequest('Limit exceeded, maybe next time.')
        else:
            API_IP.objects.filter(id=log.id).update(count=F('count') + 1)

        try:
            return super(JSONMixin, self).dispatch(request, *args, **kwargs)
        except JSONError:
            return HttpResponseBadRequest('Bad request')

    def paginate(self, obj, **kwargs):
        try:
            page = kwargs['page']
        except KeyError:
            page = 1

        paginator = Paginator(obj, self.paginate_by)
        try:
            obj = paginator.page(page)
        except EmptyPage:
            return []
        return obj.object_list


class JSONFormView(JSONMixin):
    template_name = None
    form_class = None

    def post(self, request, *args, **kwargs):
        form = self.form_class(request.POST, request.FILES)
        if form.is_valid():
            return self.form_valid(form)
        else:
            raise JSONError()

    def form_valid(self, form):
        """
        Needs to be defined in subclass
        """
        raise NotImplementedError

    def get(self, request, *args, **kwargs):
        if not self.template_name:
            return HttpResponseNotAllowed(('POST', 'OPTIONS',))
        form = self.form_class()
        return render(request, self.template_name, {'form': form})


class JsonPostView(JSONFormView):
    def post(self, request, *args, **kwargs):
        form = self.form_class(loads(request.body))
        if form.is_valid():
            return self.form_valid(form)
        else:
            raise JSONError()


def redirect_view(request, slug):
    long_url = get_object_or_404(URL, slug=slug)
    u_agent = request.META.get('HTTP_USER_AGENT', '')

    cookies = request.COOKIES
    pk_id = ''
    for key in cookies:
        if piwik_id.search(key):
            pk_id = cookies[key]
    try:
        ref = request.META.get('HTTP_REFERER', '')
    except KeyError:
        ref = ''
    cookie, pk_id = response_cookie(pk_id)
    v = Visit(remote_ip=request.META['REMOTE_ADDR'], user_agent=u_agent, referrer=ref, url=long_url, cookie=pk_id)
    v.save()

    response = HttpResponsePermanentRedirect(long_url.url)
    response.set_cookie('_pk_id.19.c40d', cookie, expires=datetime.now() + timedelta(days=1000))
    return response


class HomeView(FormView):
    template_name = 'shorten/home.html'
    form_class = URLForm

    def get_context_data(self, **kwargs):
        context = super(HomeView, self).get_context_data(**kwargs)
        context.update({'domain': Site.objects.get_current().domain})
        return context

    def form_valid(self, form):
        url = form.cleaned_data['url']
        try:
            u = URL.objects.get(url=url)
        except URL.DoesNotExist:
            u = URL(url=url, slug=None)
            u.save()

        return render(self.request, 'shorten/success.html', {'url': u, 'site': Site.objects.get_current()})

home = HomeView.as_view()


def save_url(form):
        url = form.cleaned_data['url']
        try:
            u = URL.objects.get(url=url)
        except URL.DoesNotExist:
            search = site_url.search(url)
            if search:
                slug = search.groups()[0]
                u = URL(url=url, slug=slug)
            else:
                u = URL(url=url, slug=None)
                u.save()
        return json_esponse({'url': u.url, 'short': 'http://%s/%s' % (CURRENT_DOMAIN, u.slug)})


class APIView(JSONFormView):
    form_class = URLForm

    def form_valid(self, form):
        return save_url(form)

api_view = csrf_exempt(APIView.as_view())


class ApiJsonView(JsonPostView):
    form_class = URLForm

    def form_valid(self, form):
        return save_url(form)

api_json_view = csrf_exempt(ApiJsonView.as_view())
